﻿using MedicalDatabaseContracts.Models;
using Microsoft.Extensions.Logging;
using Npgsql;
using System.Diagnostics;

namespace MedicalPostgresqlDatabase.Storages
{
    public class DoctorsStorage : AbstractPostgresqlStorage<Doctor>
    {
        private ILogger _logger;
        public DoctorsStorage(ILogger<DoctorsStorage> logger) : base("doctors", "doctor_id")
        {
            _logger = logger;
        }

        protected override Doctor CreateEntityFromReader(NpgsqlDataReader reader)
        {
            return new Doctor
            {
                Id = Convert.ToInt32(reader.GetOrdinal("doctor_id")),
                Name = Convert.ToString(reader.GetOrdinal("name")),
                Surname = Convert.ToString(reader.GetOrdinal("surname")),
                Patronymic = Convert.ToString(reader.GetOrdinal("patronymic")),
                PhoneNumber = Convert.ToString(reader.GetOrdinal("phone_number")),
                SpecializationId = Convert.ToInt32(reader.GetOrdinal("specializaton_id"))
            };
        }

        public override void Insert(Doctor item, out long elapsedMilliseconds)
        {
            using var connection = GetConnection();
            connection.Open();
            using var cmd = new NpgsqlCommand("INSERT INTO doctors (doctor_id, name, surname, patronymic, phone_number, specialization_id) VALUES ((nextval('doctors_id_seq')), @name, @surname, @patronymic, @phone_number, @specialization_id)", connection);
            cmd.Parameters.AddWithValue("@name", item.Name);
            cmd.Parameters.AddWithValue("@surname", item.Surname);
            cmd.Parameters.AddWithValue("@patronymic", item.Patronymic);
            cmd.Parameters.AddWithValue("@phone_number", item.PhoneNumber);
            cmd.Parameters.AddWithValue("@specialization_id", item.SpecializationId);
            Stopwatch stopwatch = new();
            stopwatch.Start();
            cmd.ExecuteNonQuery();
            stopwatch.Stop();
            elapsedMilliseconds = stopwatch.ElapsedMilliseconds;
        }

        public override void Update(Doctor item, out long elapsedMilliseconds)
        {
            using var connection = GetConnection();
            connection.Open();
            using var cmd = new NpgsqlCommand($"UPDATE doctors SET name = @name, surname = @surname, patronymic = @patronymic, phone_number = @phone_number, specialization_id = @specialization_id, WHERE {PRIMARY_KEY_COLUMN_NAME} = @id", connection); ;
            cmd.Parameters.AddWithValue("@id", item.Id);
            cmd.Parameters.AddWithValue("@name", item.Name);
            cmd.Parameters.AddWithValue("@surname", item.Surname);
            cmd.Parameters.AddWithValue("@patronymic", item.Patronymic);
            cmd.Parameters.AddWithValue("@phone_number", item.PhoneNumber);
            cmd.Parameters.AddWithValue("@specialization_id", item.SpecializationId);
            Stopwatch stopwatch = new();
            stopwatch.Start();
            cmd.ExecuteNonQuery();
            stopwatch.Stop();
            elapsedMilliseconds = stopwatch.ElapsedMilliseconds;
        }
    }
}
